<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Booking;
use App\Models\PerMinutePackage;
use App\Models\User;
use App\Models\PsychicCategory;
use App\Models\PsychicInfo;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Illuminate\Validation\Rule;
use Illuminate\Validation\Rules;
use Illuminate\Support\Facades\Hash;
use Twilio\Rest\Client;
use Illuminate\Support\Facades\Auth;

class PsychicController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function __construct()
    {
        $this->middleware('permission:Psychic List', ['only' => ['index']]);
        $this->middleware('permission:Create Psychic', ['only' => ['create', 'store']]);
        $this->middleware('permission:Edit Psychic', ['only' => ['edit', 'update']]);
        $this->middleware('permission:Delete Psychic', ['only' => ['destroy']]);
    }
    public function index(Request $request)
    {
        $users = User::role('Psychic')->with([
            'psychic_info.per_minute_package_chat',
            'psychic_info.per_minute_package_voice'
        ])->get();
        $psychic_categories = PsychicCategory::pluck('name', 'id')->toArray();
        return view('admin.psychic.index', compact('users', 'psychic_categories'));
    }
    public function psychic_booking($id)
    {
        $bookings = Booking::with(['user'])->where('psychic_id', $id)->get();
        return view('admin.psychic.booking', compact('bookings'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('admin.psychic.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', 'unique:' . User::class],
            'password' => ['required', Rules\Password::defaults()],
            'phone' => ['required', Rule::unique('users')->ignore($request->id)],
        ]);

        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
            'approved' => 0,
        ]);
        $psychic_info = PsychicInfo::create([
            'about' => $request->about,
            'online_status' => 'Offline',
            'commission' => 0,
            'user_id' => $user->id,
        ]);
        $role = Role::findByName('Psychic');
        $user->assignRole($role);
        $message = 'Psychic Created Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($id)
    {
        $user = User::with(['psychic_info'])->find($id);
        $per_minute_packages = PerMinutePackage::all();
        $psychic_categories = PsychicCategory::all();
        return view('admin.psychic.edit', compact('user', 'per_minute_packages', 'psychic_categories'));
    }
    /**
     * Update the specified resource in storage.
     */
    public function psychic_category_choose(Request $request)
    {
        $psychic_info = PsychicInfo::where('user_id', $request->id)->first();
        $psychic_info = $psychic_info->update([
            'psychic_category_id' => $request->psychic_category_id,
        ]);
        $message = 'Psychic Updated Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    public function update(Request $request)
    {
        $user = User::find($request->id);
        $psychic_info = PsychicInfo::where('user_id', $user->id)->first();
        // Validate the incoming request data
        $validatedData = $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'name' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', Rule::unique('users')->ignore($request->id)],
            'phone' => ['required'],
            'reading_info' => ['required'],
            'voice_call' => [
                function ($attribute, $value, $fail) use ($request, $user, $psychic_info) {
                    if ($request->voice_call && (($request->phone != $user->phone) || ($psychic_info->twilio_id == ''))) {
                        $fail("Verification of phone number is required.");
                    }
                }
            ],
        ]);
        $request_arr = [
            'name' => isset($request->approve_name) ? $psychic_info->temp_user_name : $request->name,
            'email' => $request->email,
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
            'approved' => $request->approved ? 1 : 0,
        ];
        if ($request->hasFile('profile_image')) {
            $filesName = '';
            $files = $request->file('profile_image');
            $inc = 0;
            foreach ($files as $file) {
                $type = $file->getClientOriginalExtension();
                $src = time() + $inc . '.' . $type;
                $file->move(public_path('images'), $src);
                $filesName = $src;
                $inc++;
            }
            $request_arr['image'] = $filesName;
        }
        $udpated_user = $user->update($request_arr);
        $psychic_info = $psychic_info->update([
            'about' => isset($request->approve_about) ? $psychic_info->temp_about : $request->about,
            'temp_about' => isset($request->approve_about) ? '' : $psychic_info->temp_about,
            'temp_user_name' => isset($request->approve_name) ? '' : $psychic_info->temp_user_name,
            'temp_psychic_category_id' => isset($request->approve_psychic_category_id) ? '' : $psychic_info->temp_psychic_category_id,
            'reading_info' => $request->reading_info,
            'online_status' => $request->online_status,
            'chat' => $request->chat ? 1 : 0,
            'voice_call' => $request->voice_call ? 1 : 0,
            'commission' => $request->commission,
            'per_minute_package_chat_id' => $request->per_minute_package_chat_id,
            'per_minute_package_voice_id' => $request->per_minute_package_voice_id,
            'psychic_category_id' => isset($request->approve_psychic_category_id) ? $psychic_info->temp_psychic_category_id : $request->psychic_category_id,
        ]);
        $message = 'Psychic Updated Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    public function edit_profile(Request $request)
    {
        $user = User::find($request->id);
        $psychic_info = PsychicInfo::where('user_id', $user->id)->first();
        // Validate the incoming request data
        $validatedData = $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', Rule::unique('users')->ignore($request->id)],
            'phone' => ['required', Rule::unique('users')->ignore($request->id)],
            'per_minute_package_chat_id' => ['required'],
            'per_minute_package_voice_id' => ['required'],
            'psychic_category_id' => ['required'],
            'voice_call' => [
                function ($attribute, $value, $fail) use ($request, $user, $psychic_info) {
                    if ($request->voice_call && (($request->phone != $user->phone) || ($psychic_info->twilio_id == ''))) {
                        $fail("Verification of phone number is required.");
                    }
                }
            ],
        ], [
            'per_minute_package_chat_id.required' => 'The per minute live chat package is required.',
            'per_minute_package_voice_id.required' => 'The per minute voice chat package is required.',
            'psychic_category_id.required' => 'The category is required.',
        ]);
        $request_arr = [
            'email' => $request->email,
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
        ];
        if ($request->hasFile('profile_image')) {
            $filesName = '';
            $files = $request->file('profile_image');
            $inc = 0;
            foreach ($files as $file) {
                $type = $file->getClientOriginalExtension();
                $src = time() + $inc . '.' . $type;
                $file->move(public_path('images'), $src);
                $filesName = $src;
                $inc++;
            }
            $request_arr['image'] = $filesName;
        }
        if ($psychic_info->twilio_id && $request->phone != $user->phone) {
            $sid = "ACf4a19d3132b761b06572cec63c8829f8";
            $token = "724129b0c54035005812927ff927fa41";
            $twilio = new Client($sid, $token);
            $twilio->outgoingCallerIds($psychic_info->twilio_id)->delete();
        }
        $psychic_info = $psychic_info->update([
            'temp_about' => ($psychic_info->temp_about ? $psychic_info->temp_about : $request->about != $psychic_info->about) ? $request->about : '',
            'temp_user_name' => ($psychic_info->temp_user_name ? $psychic_info->temp_user_name : $request->name != $user->name) ? $request->name : '',
            'temp_psychic_category_id' => ($psychic_info->temp_psychic_category_id ? $psychic_info->temp_psychic_category_id : $request->psychic_category_id == $psychic_info->psychic_category_id) ? '' : $request->psychic_category_id,
            'online_status' => $request->online_status,
            'chat' => $request->chat ? 1 : 0,
            'voice_call' => $request->voice_call ? 1 : 0,
            'twilio_id' => $request->phone == $user->phone ? $psychic_info->twilio_id : '',
            'per_minute_package_chat_id' => $request->per_minute_package_chat_id,
            'per_minute_package_voice_id' => $request->per_minute_package_voice_id,
        ]);
        $udpated_user = $user->update($request_arr);
        $message = 'Profile Updated Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id)
    {
        PsychicInfo::where('user_id', $id)->delete();
        User::destroy($id);
        // Optionally, you can redirect to a success page or perform other actions
        $message = 'Psychic Deleted Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    public function twilio_validation_code()
    {
        $user = Auth::user();
        // Your Twilio credentials
        $sid = "ACf4a19d3132b761b06572cec63c8829f8";
        $token = "724129b0c54035005812927ff927fa41";
        $twilio = new Client($sid, $token);
        $validation_code = '';
        $message = '';
        try {
            $validation_request = $twilio->validationRequests->create(
                $user->phone, // phone_number
                ["friendlyName" => $user->name, "StatusCallback" => "https://askandanswers.com/twilio/statuscallback"]
            );
            $validation_code = $validation_request->validationCode;
        } catch (\Throwable $th) {
            $message = str_replace('[HTTP 400] ', '', $th->getMessage());
        }
        return response()->json([
            'message' => $message,
            'validationCode' => $validation_code
        ]);
    }
    public function verify_phone()
    {
        $success = 'no';
        $sid = "ACf4a19d3132b761b06572cec63c8829f8";
        $token = "724129b0c54035005812927ff927fa41";
        $twilio = new Client($sid, $token);
        $user = Auth::user();
        do {
            $outgoingCallerIds = $twilio->outgoingCallerIds->read(
                ["phoneNumber" => $user->phone],
                20
            );

            foreach ($outgoingCallerIds as $record) {
                $success = 'yes';
                $psychic_info = PsychicInfo::where('user_id', $user->id)->first();
                $psychic_info = $psychic_info->update([
                    'twilio_id' => $record->sid,
                ]);
                break;
            }
        } while ($success == 'no');
        return response()->json([
            'success' => $success
        ]);
    }
}