<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Booking;
use App\Models\User;
use App\Models\UserInfo;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use Illuminate\Validation\Rule;
use Illuminate\Validation\Rules;
use Illuminate\Support\Facades\Hash;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function __construct()
    {
        $this->middleware('permission:User List', ['only' => ['index']]);
        $this->middleware('permission:Create User', ['only' => ['create', 'store']]);
        $this->middleware('permission:Edit User', ['only' => ['edit', 'update']]);
        $this->middleware('permission:Delete User', ['only' => ['destroy']]);
    }
    public function index(Request $request)
    {
        $users = User::role('user')->get();
        return view('admin.user.index', compact('users'));
    }

    public function user_booking($id)
    {
        $bookings = Booking::with(['psychic'])->where('user_id', $id)->get();
        return view('admin.user.booking', compact('bookings'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('admin.user.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'name' => ['required', 'string', 'lowercase', 'max:255', 'unique:' . User::class],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', 'unique:' . User::class],
            'password' => ['required', Rules\Password::defaults()],
            'phone' => ['required', Rule::unique('users')->ignore($request->id)],
        ]);

        $user = User::create([
            'name' => $request->name,
            'email' => $request->email,
            'password' => Hash::make($request->password),
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
            'approved' => 0,
        ]);
        $user_info = UserInfo::create([
            'balance' => 0,
            'user_id' => $user->id,
        ]);
        $role = Role::findByName('User');
        $user->assignRole($role);
        $message = 'User Created Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(User $user)
    {
        $user->load('user_info');
        return view('admin.user.edit', compact('user'));
    }
    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, User $user)
    {
        // Validate the incoming request data
        $validatedData = $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'name' => ['required', 'string', 'lowercase', 'max:255', Rule::unique('users')->ignore($user->id)],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', Rule::unique('users')->ignore($user->id)],
            'phone' => ['required', Rule::unique('users')->ignore($user->id)],
            'balance' => ['required'],
        ]);
        $request_arr = [
            'name' => $request->name,
            'email' => $request->email,
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
            'approved' => $request->approved ? 1 : 0,
        ];
        if ($request->hasFile('profile_image')) {
            $filesName = '';
            $files = $request->file('profile_image');
            $inc = 0;
            foreach ($files as $file) {
                $type = $file->getClientOriginalExtension();
                $src = time() + $inc . '.' . $type;
                $file->move(public_path('images'), $src);
                $filesName = $src;
                $inc++;
            }
            $request_arr['image'] = $filesName;
        }
        $udpated_user = $user->update($request_arr);
        if (!$user->hasRole('Super Admin')) {
            $user_info = UserInfo::where('user_id', $user->id)->first();
            $updated_user_info = $user_info->update([
                'balance' => $request->balance,
            ]);
        }
        $message = 'User Updated Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    public function edit_profile(Request $request)
    {
        $user = User::find($request->id);
        // Validate the incoming request data
        $validatedData = $request->validate([
            'fname' => ['required', 'string', 'max:255'],
            'lname' => ['required', 'string', 'max:255'],
            'name' => ['required', 'string', 'lowercase', 'max:255', Rule::unique('users')->ignore($user->id)],
            'email' => ['required', 'string', 'lowercase', 'email', 'max:255', Rule::unique('users')->ignore($user->id)],
            'phone' => ['required'],
        ]);
        $request_arr = [
            'name' => $request->name,
            'email' => $request->email,
            'fname' => $request->fname,
            'lname' => $request->lname,
            'phone' => $request->phone,
        ];
        if ($request->hasFile('profile_image')) {
            $filesName = '';
            $files = $request->file('profile_image');
            $inc = 0;
            foreach ($files as $file) {
                $type = $file->getClientOriginalExtension();
                $src = time() + $inc . '.' . $type;
                $file->move(public_path('images'), $src);
                $filesName = $src;
                $inc++;
            }
            $request_arr['image'] = $filesName;
        }
        $udpated_user = $user->update($request_arr);
        $message = 'Profile Updated Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
    /**
     * Remove the specified resource from storage.
     */
    public function destroy(User $user)
    {
        UserInfo::where('user_id', $user->id)->delete();
        $user->delete();
        // Optionally, you can redirect to a success page or perform other actions
        $message = 'User Deleted Successfully';
        $type = 'success';
        return redirect()->back()->with('message', $message)->with('type', $type);
    }
}